#pragma once

namespace JGL {

#define stringify(name) # name

    enum class Direction : u8 {
        None = 0,
        Vertical = 1,
        Horizontal = 2,
        Diagonal_NWSE = 3, // North West -> South East.
        Diagonal_SWNE = 4 // South West -> North East.
    };

    inline Direction operator|(Direction a, Direction b) {
        return static_cast<Direction>(static_cast<int>(a) | static_cast<int>(b));
    }

    inline bool operator&(Direction a, Direction b) {
        return (u8)a & (u8)b;
    }

    static std::string to_string(const JGL::Direction& direction) {
        return stringify(direction);
    }

    // TODO come up with a cleaner way to type out sample-rates before release.
    // :shrug:, I'm not very creative. - Redacted.
    /*enum class MSAA_SAMPLE_RATE : u8 {
        NONE = 0,
        MSAA_2X = 1,
        MSAA_4X = 2,
        MSAA_8X = 3
    };

    enum class ANISOTROPY_SAMPLE_RATE : u8 {
        NONE = 0,
        ANISOTROPY_2X = 2,
        ANISOTROPY_4X = 4,
        ANISOTROPY_8X = 8,
        ANISOTROPY_16X = 16
    };*/

    /// This enumeration is used by both MSAA and Anisotropic Filtering.
    /// However, higher rates may not be supported by one, or the other.
    /// This additionally depends on what your graphics supports.
    enum class SampleRate : u8 {
        NONE = 0, X0 = 0,
        X2 = 2,
        X4 = 4,
        X8 = 8,
        X16 = 16,
    };

    enum class FilteringMode : u8 {
        NEAREST = 0, // Fastest for 2D, Sometimes causes graphical issues.
        BILINEAR = 1, // Fast and pretty, The best for 2D.

        // Mipmapping uses approx 30% more v-ram.
        MIPMAP_NEAREST = 2, // Nearest with mipmaps. The fastest for 3D, Sometimes causes graphical issues.
        MIPMAP_BILINEAR = 3, // Bilinear with mipmaps, Fast and pretty. Uses more vram.
        MIPMAP_TRILINEAR = 4 // Prettier, But still looks weird at very steep angles.
    };

    enum class WrappingMode : u8 {
        REPEAT = 0,
        MIRRORED_REPEAT = 1,
        CLAMP_TO_EDGE = 2,
        CLAMP_TO_BORDER = 3 // Effectively the same as clamp_to_edge
    };


    enum class ColorFormat : bool { RGB = false, RGBA = true };

    static std::string to_string(const JGL::SampleRate& sample_rate) {
        return stringify(sample_rate);
    }
    static int to_int(const JGL::SampleRate& sample_rate) {
        return (int)sample_rate;
    }
}
