//
// Created by dawsh on 1/17/24.
//

#include <glad/glad.h>
#include <JGL/JGL.h>
#include <JGL/logger/logger.h>
#include "renderer/OpenGL/internals/internals.h"

namespace JGL {
    using namespace J3ML;

    bool Init(const Vector2i& ws, float fovY, float far_plane) {
        gladLoadGL();
        if (!MeetsRequirements()) {
            Logger::Fatal("The graphics driver does not meet the minimum requirements to run this program.");
            return false;
        }

        InitTextEngine();
        Fonts::Init();
        ShapeCache::Init();
        JGL::window_size = ws;
        J3D::fov = fovY;
        J3D::far_plane = far_plane;
        return true;
    }

    void Update(const Vector2i& ws) {
        if (state_stack.Size())
            Logger::Error("You shouldn't be running JGL::Update while inside a J2D or J3D context.");

        JGL::window_size = ws;
        glViewport(0, 0, window_size.x, window_size.y);
    }

    bool MeetsRequirements() {
        if (!GLAD_GL_VERSION_2_1)
            return false;
        if (!GLAD_GL_ARB_framebuffer_object)
            return false;
        if (!GLAD_GL_ARB_depth_texture)
            return false;
        if (!GLAD_GL_ARB_shadow)
            return false;
        return true;
    }
}
